import { useMutation, useQuery, useSuspenseQuery, } from "@tanstack/react-query";
export default function createClient(client) {
    return {
        useQuery: (method, path, ...[init, options, queryClient]) => {
            return useQuery({
                queryKey: [method, path, init],
                queryFn: async () => {
                    const mth = method.toUpperCase();
                    const fn = client[mth];
                    const { data, error } = await fn(path, init);
                    if (error || !data) {
                        throw error;
                    }
                    return data;
                },
                ...options,
            }, queryClient);
        },
        useSuspenseQuery: (method, path, ...[init, options, queryClient]) => {
            return useSuspenseQuery({
                queryKey: [method, path, init],
                queryFn: async () => {
                    const mth = method.toUpperCase();
                    const fn = client[mth];
                    const { data, error } = await fn(path, init);
                    if (error || !data) {
                        throw error;
                    }
                    return data;
                },
                ...options,
            }, queryClient);
        },
        useMutation: (method, path, options, queryClient) => {
            return useMutation({
                mutationKey: [method, path],
                mutationFn: async (init) => {
                    const mth = method.toUpperCase();
                    const fn = client[mth];
                    const { data, error } = await fn(path, init);
                    if (error || !data) {
                        throw error;
                    }
                    return data;
                },
                ...options,
            }, queryClient);
        },
    };
}
//# sourceMappingURL=index.js.map